/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.plugin.view;

import java.util.List;
import java.util.regex.Pattern;								// 6.4.3.3 (2016/03/04)

import org.opengion.hayabusa.common.HybsSystemException;
// import org.opengion.hayabusa.db.DBTableModel;
import org.opengion.fukurou.util.StringUtil;
import org.opengion.hayabusa.html.TableFormatter;

/**
 * ヘッダ、フッタ、ボディを指定して作成する、自由レイアウトが可能な、カスタムテーブル表示クラスです。
 * このクラスは、ViewForm_HTMLFormatTable クラスの代替えとしても使用できます。
 * その場合は、thead のみ指定すれば、同じフォームが tbody にも適用されます。
 * これは、まさに、ViewForm_HTMLFormatTable と同じです。
 *
 * AbstractViewForm により、setter/getterメソッドのデフォルト実装を提供しています。
 * 各HTMLのタグに必要な setter/getterメソッドのみ，追加定義しています。
 *
 * AbstractViewForm を継承している為,ロケールに応じたラベルを出力させる事が出来ます。
 *
 * @og.group 画面表示
 *
 * @version  4.0
 * @author	 Kazuhiko Hasegawa
 * @since    JDK5.0,
 */
public class ViewForm_HTMLCustomTable extends ViewForm_HTMLTable	{
	/** このプログラムのVERSION文字列を設定します。	{@value} */
	private static final String VERSION = "6.8.1.1 (2017/07/22)" ;

	// 6.4.3.3 (2016/03/04) class属性(ColumnのDBType)置換で、td属性に[カラム]があると、誤ってそちらがセットされてしまう対応。
	// ※ 正規表現の「.」は、改行を除く任意の文字なので、改行を含みたい場合は、「[\\s\\S]」とします。
//	private static final Pattern TD_BODY = Pattern.compile( "[\\s\\S]*<td[^>]*>" );				// tdタグが完了して、BODY部分の解析に入る。
	private static final Pattern TD_BODY = Pattern.compile( "[\\s\\S]*<td[^>]*>[\\s]*" );		// 6.4.4.2 (2016/04/01) tdタグが完了して、BODY部分の解析に入る。
	private static final Pattern TD_END  = Pattern.compile( "[^<]*>[^<>]*" );					// 何らかのタグが完了して、BODY部分の解析に入る。

	// 3.5.4.0 (2003/11/25) TableFormatter クラス追加
	// 4.3.1.0 (2008/09/08) protectedに変更
	/** ヘッダーフォーマット変数 */
	protected TableFormatter 		headerFormat	;
	/** ボディーフォーマット配列変数 */
	protected TableFormatter[]		bodyFormats		;
	/** フッターフォーマット変数 */
	protected TableFormatter		footerFormat	;
	/** ボディーフォーマット数 */
	protected int					bodyFormatsCount;

	// 3.5.4.6 (2004/01/30) 初期値変更
	/** ボディーフォーマット最大数 初期値:{@value} */
	protected static final int BODYFORMAT_MAX_COUNT = 15;		// 5.7.6.2 (2014/05/16)

	/**
	 * デフォルトコンストラクター
	 *
	 * @og.rev 6.4.2.0 (2016/01/29) PMD refactoring. Each class should declare at least one constructor.
	 */
	public ViewForm_HTMLCustomTable() { super(); }		// これも、自動的に呼ばれるが、空のメソッドを作成すると警告されるので、明示的にしておきます。

	/**
	 * DBTableModel から HTML文字列を作成して返します。
	 * startNo(表示開始位置)から、pageSize(表示件数)までのView文字列を作成します。
	 * 表示残りデータが pageSize 以下の場合は,残りのデータをすべて出力します。
	 *
	 * @og.rev 3.5.0.0 (2003/09/17) BODY要素の noClass 属性を追加。
	 * @og.rev 3.5.0.0 (2003/09/17) &lt;tr&gt;属性は、元のフォーマットのまま使用します。
	 * @og.rev 3.5.2.0 (2003/10/20) ヘッダー繰り返し属性( headerSkipCount )を採用
	 * @og.rev 3.5.3.1 (2003/10/31) skip属性を採用。headerLine のキャッシュクリア
	 * @og.rev 3.5.4.0 (2003/11/25) TableFormatter クラスを使用するように変更。
	 * @og.rev 3.5.5.0 (2004/03/12) systemFormat(例：[KEY.カラム名]形式等)の対応
	 * @og.rev 3.5.5.0 (2004/03/12) No 欄そのものの作成判断ロジックを追加
	 * @og.rev 3.5.5.7 (2004/05/10) [#カラム名] , [$カラム名] に対応
	 * @og.rev 3.5.6.0 (2004/06/18) '!' 値のみ 追加 既存の '$' は、レンデラー
	 * @og.rev 3.5.6.2 (2004/07/05) makeFormat 処理をgetTableHead メソッドから移動
	 * @og.rev 3.5.6.4 (2004/07/16) ヘッダーとボディー部をJavaScriptで分離
	 * @og.rev 4.0.0.0 (2005/01/31) 新規作成(getColumnClassName ⇒ getColumnDbType)
	 * @og.rev 3.7.0.3 (2005/03/01) getBgColorCycleClass に、選択行マーカーを採用
	 * @og.rev 4.3.1.0 (2008/09/08) フォーマットが設定されていない場合のエラー追加・編集行のみを表示する属性(isSkipNoEdit)追加
	 * @og.rev 4.3.3.0 (2008/10/01) noTransition属性対応
	 * @og.rev 4.3.7.4 (2009/07/01) tbodyタグの入れ子を解消(FireFox対応)
	 * @og.rev 6.2.0.0 (2015/02/27) フォーマット系の noDisplay 対応
	 * @og.rev 6.4.3.3 (2016/03/04) class属性(ColumnのDBType)置換で、td属性に[カラム]があると、誤ってそちらがセットされてしまう対応。
	 * @og.rev 6.4.3.4 (2016/03/11) tdに、[カラム]が無いケースで、次の[カラム]のクラス属性が、前方すべてのtdにセットされてしまう対応。
	 * @og.rev 6.4.4.2 (2016/04/01) TableFormatterのタイプ別値取得処理の共通部をまとめる。
	 * @og.rev 6.4.5.0 (2016/04/08) メソッド変更( getColumnDbType(int) → getClassName(int) )
	 * @og.rev 6.8.1.1 (2017/07/22) ckboxTD変数は、&lt;td&gt; から &lt;td に変更します(タグの最後が記述されていない状態でもらう)。
	 * @og.rev 6.8.1.1 (2017/07/22) 行番号のtdに、個別に class="S9" を追加するための useS9 変数を追加。
	 *
	 * @param  startNo	  表示開始位置
	 * @param  pageSize   表示件数
	 *
	 * @return	DBTableModelから作成された HTML文字列
	 * @og.rtnNotNull
	 */
	@Override
	public String create( final int startNo, final int pageSize )  {
		if( getRowCount() == 0 ) { return ""; }	// 暫定処置

		// 4.3.1.0 (2008/09/08)
		if( headerFormat == null ) {
			final String errMsg = "ViewTagで canUseFormat() = true の場合、Formatter は必須です。";
			throw new HybsSystemException( errMsg );
		}

		headerLine	 = null;		// 3.5.3.1 (2003/10/31) キャッシュクリア

		final int lastNo = getLastNo( startNo, pageSize );
		final int blc = getBackLinkCount();
		final int hsc = getHeaderSkipCount();		// 3.5.2.0 (2003/10/20)
		int hscCnt = 1;						// 3.5.2.0 (2003/10/20)

		headerFormat.makeFormat( getDBTableModel() );	// 3.5.6.2 (2004/07/05) 移動
		// 6.2.0.0 (2015/02/27) フォーマット系の noDisplay 対応
		setFormatNoDisplay( headerFormat );

		final StringBuilder out = new StringBuilder( BUFFER_LARGE )
			.append( getCountForm( startNo,pageSize ) )
			.append( getHeader() );

		if( bodyFormatsCount == 0 ) {
			bodyFormats[0] = headerFormat ;
			bodyFormatsCount ++ ;
		}
		else {
			for( int i=0; i<bodyFormatsCount; i++ ) {
				bodyFormats[i].makeFormat( getDBTableModel() );
				// 6.2.0.0 (2015/02/27) フォーマット系の noDisplay 対応
				setFormatNoDisplay( bodyFormats[i] );
			}
		}

		int bgClrCnt = 0;
		for( int row=startNo; row<lastNo; row++ ) {
			if( isSkip( row ) || isSkipNoEdit( row ) ) { continue; }				// 4.3.1.0 (2008/09/08)
			for( int i=0; i<bodyFormatsCount; i++ ) {
				final TableFormatter bodyFormat = bodyFormats[i];
				if( ! bodyFormat.isUse( row,getDBTableModel() ) ) { continue; }		// 3.5.4.0 (2003/11/25)
				out.append("<tbody").append( getBgColorCycleClass( bgClrCnt++,row ) );
				if( isNoTransition() ) { // 4.3.3.0 (2008/10/01)
					out.append( getHiddenRowValue( row ) );
				}
				out.append('>')		// 6.0.2.5 (2014/10/31) char を append する。
					.append( bodyFormat.getTrTag() );

				// 3.5.5.0 (2004/03/12) No 欄そのものの作成判断追加
				if( isNumberDisplay() ) {
//					final String ckboxTD = "<td" + bodyFormat.getRowspan() + ">";
					final String ckboxTD = "<td" + bodyFormat.getRowspan();		// 6.8.1.1 (2017/07/22)
//					out.append( makeCheckbox( ckboxTD,row,blc ) );
					out.append( makeCheckbox( ckboxTD,row,blc,true ) );			// 6.8.1.1 (2017/07/22) 行番号のtdに、個別に class="S9" を追加
				}

				int cl = 0;
				boolean isTdBody = false;										// 6.4.3.3 (2016/03/04) tdBody処理を行ってよいかどうかのフラグ
				for( ; cl<bodyFormat.getLocationSize(); cl++ ) {
					String fmt = bodyFormat.getFormat(cl);
					final int loc = bodyFormat.getLocation(cl);					// 3.5.5.0 (2004/03/12)
					if( ! bodyFormat.isNoClass() && loc >= 0 ) {				// 3.5.5.7 (2004/05/10)
						if( isTdBody && TD_END.matcher( fmt ).matches() ) {		// 6.4.3.3 (2016/03/04) tdBody許可があり、TDが閉じた場合。
							isTdBody = false;
							final int idx = out.lastIndexOf( "<td" );
							if( idx >= 0 ) { 
//								final String tdclass = " class=\"" + getColumnDbType(loc) + "\" ";
								final String tdclass = " class=\"" + getClassName(loc) + "\" ";			// 6.4.5.0 (2016/04/08)
								out.insert( idx+3, tdclass );					// +3 は、"<td" の後ろという意味。fmtでなくそれ以前に戻る必要がある。
							}
						}

						if( TD_BODY.matcher( fmt ).matches() ) {				// 6.4.3.3 (2016/03/04) TDが閉じた場合。
							// 6.4.3.4 (2016/03/11) tdに、[カラム]が無いケースで、次の[カラム]のクラス属性が、前方すべてのtdにセットされてしまう対応。
							// 上の処理と統一しなければいけないが、とりあえず暫定対応
							final int idx = fmt.lastIndexOf( "<td" );
							if( idx >= 0 ) {	// matchしてるので、あるはず
//								final String tdclass = " class=\"" + getColumnDbType(loc) + "\" ";
								final String tdclass = " class=\"" + getClassName(loc) + "\" ";			// 6.4.5.0 (2016/04/08)
								fmt = fmt.substring( 0,idx+3 ) + tdclass + fmt.substring( idx+3 ) ;
							}
//							final String tdclass = "<td class=\"" + getColumnDbType(loc) + "\" ";
//							fmt = StringUtil.replace( fmt ,"<td", tdclass );	// 6.2.0.1 (2015/03/06)
							isTdBody = false;									// 6.4.3.3 (2016/03/04) これは、要らない･･･はず。
						}
						else {
							isTdBody = true;									// TDが閉じていない。
						}
					}
					out.append( fmt );			// 3.5.0.0

					// 3.5.5.7 (2004/05/10) #,$ 対応
					if( loc >= 0 ) {
						// 6.4.4.2 (2016/04/01) 処理の共通部をまとめる。
						out.append( getTypeCaseValue( bodyFormat.getType(cl),row,loc ) );
//						switch( bodyFormat.getType(cl) ) {
//							case '#' : out.append( getColumnLabel(loc) );		break;
//							case '$' : out.append( getRendererValue(row,loc) );	break;
//							case '!' : out.append( getValue(row,loc) );			break;
//							default  : out.append( getValueLabel(row,loc) );	break;
//						}
					}
					else {
						out.append( bodyFormat.getSystemFormat(row,loc) );
					}
				}
				out.append( bodyFormat.getFormat(cl) )
					.append("</tbody>").append( CR );
			}

		// 3.5.2.0 (2003/10/20) ヘッダー繰り返し属性( headerSkipCount )を採用
			if( hsc > 0 && hscCnt % hsc == 0 ) {
				out.append("<tbody class=\"row_h\" >")
					.append( getHeadLine() )
					.append("</tbody>");
				hscCnt = 1;
			}
			else {
				hscCnt ++ ;
			}
		}

		if( footerFormat != null ) {
		// 6.3.9.0 (2015/11/06) 引数にTableFormatterを渡して、処理の共有化を図る。
//			out.append( getTableFoot() );
			out.append( getTableFoot( footerFormat ) );
		}

		out.append("</table>").append( CR )
			.append( getScrollBarEndDiv() );	// 3.8.0.3 (2005/07/15)

		return out.toString();
	}

	/**
	 * 内容をクリア(初期化)します。
	 *
	 * @og.rev 3.1.1.0 (2003/03/28) 同期メソッド(synchronized付き)を非同期に変更する。
	 * @og.rev 3.5.0.0 (2003/09/17) Noカラムに、表示を全て消せるように、class 属性を追加。
	 * @og.rev 3.5.4.0 (2003/11/25) TableFormatter クラスを使用するように変更。
	 *
	 */
	@Override
	public void clear() {
		super.clear();
		headerFormat			= null;
		bodyFormats				= null;
		footerFormat			= null;
		bodyFormatsCount		= 0;
	}

	/**
	 * DBTableModel から テーブルのタグ文字列を作成して返します。
	 *
	 * @og.rev 3.5.0.0 (2003/09/17) &lt;tr&gt;属性は、元のフォーマットのまま使用します。
	 * @og.rev 3.5.1.0 (2003/10/03) Noカラムに、numberType 属性を追加
	 * @og.rev 3.5.2.0 (2003/10/20) ヘッダー繰り返し部をgetHeadLine()へ移動
	 * @og.rev 3.5.3.1 (2003/10/31) VERCHAR2 を VARCHAR2 に修正。
	 * @og.rev 3.5.4.0 (2003/11/25) TableFormatter クラスを使用するように変更。
	 * @og.rev 3.5.5.0 (2004/03/12) No 欄そのものの作成判断ロジックを追加
	 * @og.rev 3.5.6.2 (2004/07/05) makeFormat 処理をcreate メソッドの頭に移動
	 * @og.rev 3.5.6.5 (2004/08/09) thead に、id="header" を追加
	 * @og.rev 4.0.0.0 (2005/01/31) DBColumn の 属性(CLS_NM)から、DBTYPEに変更
	 * @og.rev 5.9.1.2 (2015/10/23) 自己終了警告対応
	 * @og.rev 6.4.4.1 (2016/03/18) NUMBER_DISPLAYを、static final 定数化します。
	 * @og.rev 6.4.9.0 (2016/07/23) colgroupのHTML5対応(No欄)
	 * @og.rev 6.4.9.1 (2016/08/05) colgroupのHTML5対応(No欄)時の対応ミス修正
	 * @og.rev 6.8.1.0 (2017/07/14) HTML5対応ヘッダー出力設定時に、ブラウザを互換設定したときの対応。
	 *
	 * @return	テーブルのタグ文字列
	 * @og.rtnNotNull
	 */
	@Override
	protected String getTableHead() {
		final StringBuilder buf = new StringBuilder( BUFFER_MIDDLE );

		// 3.5.5.0 (2004/03/12) No 欄そのものの作成判断追加
		if( isNumberDisplay() ) {
			// 6.4.9.0 (2016/07/23) colgroupのHTML5対応(No欄)
//			if( useIE7Header ) { 
				buf.append( NUMBER_DISPLAY );		// 6.8.1.0 (2017/07/14) HTML5ネイティブ時でも、出力します。
//			}
//			else {
			// 6.8.1.0 (2017/07/14) HTML5対応ヘッダー出力設定時に、ブラウザを互換設定したときの対応。
			if( !useIE7Header ) { 
				buf.append( "<style type=\"text/css\">" ).append( CR );
				makeNthChild( buf,2,"BIT" );
				makeNthChild( buf,3,"S9"  );
				buf.append( "</style>" ).append( CR );		// 6.4.9.1 (2016/08/05)
			}
			// 5.9.1.2 (2015/10/23) 自己終了警告対応
//			buf.append("<colgroup class=\"X\" /><colgroup class=\"BIT\" /><colgroup class=\"S9\" />")
			// 6.4.4.1 (2016/03/18) NUMBER_DISPLAYを、static final 定数化します。
//			buf.append("<colgroup class=\"X\" ><!-- --></colgroup><colgroup class=\"BIT\" ><!-- --></colgroup><colgroup class=\"S9\" ><!-- --></colgroup>")
//				.append(CR);
//			buf.append( NUMBER_DISPLAY );
		}

		// 3.5.2.0 (2003/10/20) ヘッダー繰り返し部をgetHeadLine()へ移動
		buf.append("<thead id=\"header\">").append( CR )		// 3.5.6.5 (2004/08/09)
			.append( getHeadLine() )
			.append("</thead>").append( CR );

		return buf.toString();
	}

	/**
	 * ヘッダー繰り返し部を、getTableHead()メソッドから分離。
	 *
	 * @og.rev 3.5.6.2 (2004/07/05) HTMLCustomTableScrollBar 用に新規作成
	 * @og.rev 6.1.2.0 (2015/01/24) キャッシュを返すのを、#getHeadLine() に移動。
	 *
	 * @return	テーブルのタグ文字列
	 * @og.rtnNotNull
	 */
	@Override
	protected String getHeadLine() {
		if( headerLine == null ) {					// キャッシュになければ、設定する。
			headerLine = getHeadLine( "<th" + headerFormat.getRowspan() ) ;
		}

		return headerLine ;
	}

	/**
	 * ヘッダー繰り返し部を、getTableHead()メソッドから分離。
	 *
	 * @og.rev 3.5.2.0 (2003/10/20) 新規作成
	 * @og.rev 3.5.4.0 (2003/11/25) TableFormatter クラスを使用するように変更。
	 * @og.rev 3.5.4.3 (2004/01/05) useCheckControl 属性の機能を追加
	 * @og.rev 3.5.4.6 (2004/01/30) numberType="none" 時の処理を追加(Noラベルを出さない)
	 * @og.rev 3.5.4.7 (2004/02/06) ヘッダーにソート機能用のリンクを追加します。
	 * @og.rev 3.5.5.0 (2004/03/12) systemFormat(例：[KEY.カラム名]形式等)の対応
	 * @og.rev 3.5.5.0 (2004/03/12) No 欄そのものの作成判断ロジックを追加
	 * @og.rev 3.5.6.2 (2004/07/05) HTMLCustomTableScrollBar 用に引数追加
	 * @og.rev 3.7.0.1 (2005/01/31) 全件チェックコントロール処理変更
	 * @og.rev 6.1.2.0 (2015/01/24) キャッシュを返すのを、#getHeadLine() に移動。
	 * @og.rev 6.4.3.4 (2016/03/11) ヘッダーでもTableFormatterのType(#,$,!)に対応した値を出すようにする。
	 * @og.rev 6.4.4.2 (2016/04/01) TableFormatterのタイプ別値取得処理の共通部をまとめる。
	 *
	 * @param	thTag タグの文字列
	 *
	 * @return	テーブルのタグ文字列
	 * @og.rtnNotNull
	 */
	@Override
	protected String getHeadLine( final String thTag ) {

		final StringBuilder buf = new StringBuilder( BUFFER_MIDDLE );

		buf.append( headerFormat.getTrTag() ).append( CR );

		// 3.5.5.0 (2004/03/12) No 欄そのものの作成判断追加
		if( isNumberDisplay() ) {
			// 6.1.2.0 (2015/01/24) thTag に、headerFormat.getRowspan() を含ませて受け取る。
			if( isUseCheckControl() && "checkbox".equals( getSelectedType() ) ) {
				buf.append( thTag ).append( "></th>" )
					.append( thTag ).append( '>' ).append( getAllCheckControl() ).append( "</th>" )	// 6.0.2.5 (2014/10/31) char を append する。
					.append( thTag ).append( '>' ).append( getNumberHeader()    ).append( "</th>" );	// 6.0.2.5 (2014/10/31) char を append する。
			}
			else {
				buf.append( thTag ).append( " colspan=\"3\">" ).append( getNumberHeader() ).append( "</th>" );		// 6.0.2.5 (2014/10/31) char を append する。
			}
		}

		int cl = 0;
		for( ; cl<headerFormat.getLocationSize(); cl++ ) {
			buf.append( StringUtil.replace( headerFormat.getFormat(cl) ,"td","th" ));
			final int loc = headerFormat.getLocation(cl);
			// 6.4.3.4 (2016/03/11) ヘッダーでもTableFormatterのType(#,$,!)に対応した値を出すようにする。
	//		if( loc >= 0 ) { buf.append( getSortedColumnLabel(loc) ); }
			if( loc >= 0 ) {
				// 6.4.4.2 (2016/04/01) 処理の共通部をまとめる。
				buf.append( getTypeCaseValue( headerFormat.getType(cl),-1,loc ) );
//				switch( headerFormat.getType(cl) ) {
//					case '#' : buf.append( getColumnLabel(loc) );		break;
//					case '$' : buf.append( getRendererValue(0,loc) );	break;
//					case '!' : buf.append( getValue(0,loc) );			break;
//					default  : buf.append( getSortedColumnLabel(loc) );	break;
//				}
			}
		}
		buf.append( StringUtil.replace( headerFormat.getFormat(cl) ,"td","th" ) ).append( CR );

		return buf.toString();				// 6.1.2.0 (2015/01/24)
	}

//	/**
//	 * DBTableModel から テーブルのタグ文字列を作成して返します。
//	 *
//	 * @og.rev 3.5.0.0 (2003/09/17) &lt;tr&gt;属性は、元のフォーマットのまま使用します。
//	 * @og.rev 3.5.1.0 (2003/10/03) Noカラムに、numberType 属性を追加
//	 * @og.rev 3.5.4.0 (2003/11/25) TableFormatter クラスを使用するように変更。
//	 * @og.rev 3.5.4.7 (2004/02/06) ヘッダーにソート機能用のリンクを追加します。
//	 * @og.rev 3.5.5.0 (2004/03/12) systemFormat(例：[KEY.カラム名]形式等)の対応
//	 * @og.rev 3.5.5.0 (2004/03/12) No 欄そのものの作成判断ロジックを追加
//	 * @og.rev 6.2.0.0 (2015/02/27) フォーマット系の noDisplay 対応
//	 * @og.rev 6.3.9.0 (2015/11/06) 引数にTableFormatterを渡して、処理の共有化を図る。
//	 *
//	 * @return	テーブルのタグ文字列
//	 * @og.rtnNotNull
//	 */
//	protected String getTableFoot() {
//		footerFormat.makeFormat( getDBTableModel() );
//		// 6.2.0.0 (2015/02/27) フォーマット系の noDisplay 対応
//		setFormatNoDisplay( footerFormat );
//
//		final StringBuilder buf = new StringBuilder( BUFFER_MIDDLE )
//			.append("<tfoot>").append( CR )
//			.append( footerFormat.getTrTag() ).append( CR );
//
//		// 3.5.5.0 (2004/03/12) No 欄そのものの作成判断追加
//		if( isNumberDisplay() ) {
//			buf.append("<th colspan=\"3\"")
//				.append( footerFormat.getRowspan() )
//				.append("></th>");
//		}
//
//		int cl = 0;
//		for( ; cl < footerFormat.getLocationSize(); cl++ ) {
//			final int loc = footerFormat.getLocation(cl);
//			if( loc >= 0 ) { buf.append( getSortedColumnLabel(loc) ); }
//		}
//		buf.append( footerFormat.getFormat(cl) ).append( CR )
//			.append("</tfoot>").append( CR );
//
//		return buf.toString();
//	}

	/**
	 * フォーマットを設定します。
	 *
	 * @og.rev 3.5.4.0 (2003/11/25) 新規作成
	 * @og.rev 3.5.4.4 (2004/01/16) 配列の最大数を変更
	 * @og.rev 3.5.5.5 (2004/04/23) headerFormat が定義されていない場合はエラー
	 *
	 * @param	list	TableFormatterのリスト
	 */
	@Override
	public void setFormatterList( final List<TableFormatter> list ) {		// 4.3.3.6 (2008/11/15) Generics警告対応
		bodyFormats = new TableFormatter[BODYFORMAT_MAX_COUNT];

		bodyFormatsCount = 0;
		for( int i=0; i<list.size(); i++ ) {
			final TableFormatter format = list.get( i );		// 4.3.3.6 (2008/11/15) Generics警告対応

			switch( format.getFormatType() ) {
				case TYPE_HEAD : headerFormat = format; break;
				case TYPE_BODY : bodyFormats[bodyFormatsCount++] = format; break;
				case TYPE_FOOT : footerFormat = format; break;
				default : final String errMsg = "FormatterType の定義外の値が指定されました。";
				// 4.3.4.4 (2009/01/01)
						  throw new HybsSystemException( errMsg );
			}
		}

		// 3.5.5.5 (2004/04/23) headerFormat が定義されていない場合はエラー
		if( headerFormat == null ) {
			final String errMsg = "h:thead タグの、フォーマットの指定は必須です。";
			throw new HybsSystemException( errMsg );
		}
	}

	/**
	 * フォーマットメソッドを使用できるかどうかを問い合わせます。
	 *
	 * @return  使用可能(true)/ 使用不可能 (false)
	 */
	@Override
	public boolean canUseFormat() {
		return true;
	}

	/**
	 * ビューで表示したカラムの一覧をCSV形式で返します。
	 *
	 * @og.rev 5.1.6.0 (2010/05/01) 新規追加
	 * @og.rev 6.2.0.1 (2015/03/06) TableFormatter#getLocation(int)の有効判定
	 * @og.rev 6.4.3.4 (2016/03/11) getViewClms(TableFormatter) を使用して表示されたカラム一覧を求めます。
	 *
	 * @return	ビューで表示したカラムの一覧
	 * @og.rtnNotNull
	 */
	@Override
	public String getViewClms() {
		return getViewClms( headerFormat );

//		final DBTableModel table = getDBTableModel();
//		final StringBuilder buf = new StringBuilder( BUFFER_MIDDLE );
//		for( int i=0; i<headerFormat.getLocationSize(); i++ ) {
//			if( buf.length() > 0 ) { buf.append( ',' ); }
//			// 6.2.0.1 (2015/03/06) TableFormatter#getLocation(int)の有効判定
//			final int loc = headerFormat.getLocation(i);
//			if( loc >= 0 ) { buf.append( table.getColumnName( loc ) ); }
//
//		}
//		return buf.toString();
	}
}
